#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>


import unittest

from datetime import datetime
from unittest.mock import patch, mock_open

from sirmordred.utils.healthcheck import match_error_string, read_cache_file


class TestHealthCheck(unittest.TestCase):
    """Task tests for healthcheck.py methods"""

    def test_match_error(self):
        """Test match_error_string method in charge of finding error patterns in log lines"""

        file_path = './data/mordred.log'
        match_string = 'Exception in Task Manager'

        time_a = datetime.strptime('2019-11-27 13:50:50,425', '%Y-%m-%d %H:%M:%S,%f')
        time_b = datetime.strptime('2019-11-27 13:50:52,663', '%Y-%m-%d %H:%M:%S,%f')
        self.assertTrue(match_error_string(file_path, time_a, time_b, match_string))

        time_a = datetime.strptime('2019-11-27 13:50:50,429', '%Y-%m-%d %H:%M:%S,%f')
        self.assertFalse(match_error_string(file_path, time_a, time_b, match_string))

        time_a = datetime.strptime('2019-12-01 13:50:50,425', '%Y-%m-%d %H:%M:%S,%f')
        time_b = datetime.strptime('2019-12-31 13:50:52,663', '%Y-%m-%d %H:%M:%S,%f')
        self.assertFalse(match_error_string(file_path, time_a, time_b, match_string))

    def _read_healthcheck_cache_file(name):
        """Auxiliary function to read files from ./data used by the tests"""

        file_path = './data/' + name
        with open(file_path, 'r') as f:
            return f.read()

    @patch('builtins.open', mock_open(read_data=_read_healthcheck_cache_file('healthcheck_cache_valid.json')))
    def test_read_cache_file(self):
        """Test read_cache_file with valid file"""

        a, b = read_cache_file()
        self.assertFalse(a)
        self.assertEqual(b, datetime.strptime('2019-11-27 17:27:08,602172', '%Y-%m-%d %H:%M:%S,%f'))

    @patch('builtins.open', mock_open(read_data=_read_healthcheck_cache_file('healthcheck_cache_wrong_key.json')))
    def test_read_invalid_cache_file(self):
        """Test read_cache_file with a file with one wrong key"""

        a, b = read_cache_file()
        self.assertTrue(a)
        self.assertIsNone(b)

    @patch('builtins.open', mock_open(read_data=_read_healthcheck_cache_file('healthcheck_cache_invalid.json')))
    def test_read_invalid_json_cache_file(self):
        """Test read_cache_file with an invalid JSON file"""

        a, b = read_cache_file()
        self.assertTrue(a)
        self.assertIsNone(b)


if __name__ == "__main__":
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>

import sys
import unittest
import unittest.mock

import httpretty
from urllib.parse import urljoin

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sirmordred.config import Config
from sirmordred.task_panels import (KIBANA_SETTINGS_URL,
                                    TaskPanels,
                                    TaskPanelsMenu)

CONF_FILE = 'test.cfg'


class MockedTaskPanels(TaskPanels):
    VERSION = "5.6.0"

    def __init__(self, conf):
        super().__init__(conf)

    def es_version(self, url):
        return MockedTaskPanels.VERSION.split(".")[0]

    def create_dashboard(self, panel_file, data_sources=None, strict=True):
        return


def check_import_dashboard_stackexchange(elastic_url, import_file, es_index=None,
                                         data_sources=None, add_vis_studies=False, strict=False):
    """ Check that stackexchange data sources adds also stackoverflow
        data source which is the name used in panels """
    if "stackexchange" in data_sources and "stackoverflow" not in data_sources:
        raise RuntimeError('stackexchange present but stackoverflow no in data sources')


class TestTaskPanels(unittest.TestCase):
    """TaskPanels tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        config = Config(CONF_FILE)
        task = TaskPanels(config)

        self.assertEqual(task.config, config)

    @unittest.mock.patch('sirmordred.task_panels.import_dashboard', side_effect=check_import_dashboard_stackexchange)
    @unittest.mock.patch('sirmordred.task_panels.get_dashboard_name')
    def test_create_dashboard_stackexchange(self, mock_get_dashboard_name, mock_import_dashboard):
        """ Test the creation of a dashboard which includes stackexchange in data sources """
        mock_get_dashboard_name.return_value = ''

        config = Config(CONF_FILE)
        task = TaskPanels(config)

        panel_file = 'panels/json/stackoverflow.json'
        task.create_dashboard(panel_file, data_sources=["stackexchange"])

    def test_create_dashboard_multi_ds_kibiter_6(self):
        """ Test the creation of dashboards with filtered data sources """

        config = Config(CONF_FILE)
        es_url = config.conf['es_enrichment']['url']
        es_kibana_url = urljoin(es_url + "/", '.kibana')
        kibiter_api_url = urljoin(config.conf['panels']['kibiter_url'], KIBANA_SETTINGS_URL)
        kibiter_defaultIndex_url = kibiter_api_url + '/defaultIndex'
        kibiter_timePicker_url = kibiter_api_url + '/timepicker:timeDefaults'

        headers = {
            "Content-Type": "application/json",
            "kbn-xsrf": "true"
        }

        httpretty.register_uri(httpretty.GET,
                               es_kibana_url,
                               body={},
                               status=200)

        httpretty.register_uri(httpretty.POST,
                               kibiter_defaultIndex_url,
                               body={},
                               status=200,
                               forcing_headers=headers)

        httpretty.register_uri(httpretty.POST,
                               kibiter_timePicker_url,
                               body={},
                               status=200,
                               forcing_headers=headers)

        MockedTaskPanels.VERSION = '6.1.0'
        task = MockedTaskPanels(config)
        httpretty.enable(allow_net_connect=True)
        task.execute()
        httpretty.disable()

    def test_create_dashboard_multi_ds_kibiter_6_hhtp_error(self):
        """ Test the creation of dashboards with filtered data sources """

        config = Config(CONF_FILE)
        es_url = config.conf['es_enrichment']['url']
        es_kibana_url = urljoin(es_url + "/", '.kibana')
        kibiter_api_url = urljoin(config.conf['panels']['kibiter_url'], KIBANA_SETTINGS_URL)
        kibiter_defaultIndex_url = kibiter_api_url + '/defaultIndex'
        kibiter_timePicker_url = kibiter_api_url + '/timepicker:timeDefaults'

        headers = {
            "Content-Type": "application/json",
            "kbn-xsrf": "true"
        }

        httpretty.register_uri(httpretty.GET,
                               es_kibana_url,
                               body='{}',
                               status=200)

        httpretty.register_uri(httpretty.POST,
                               kibiter_defaultIndex_url,
                               body='{}',
                               status=200,
                               forcing_headers=headers)

        httpretty.register_uri(httpretty.POST,
                               kibiter_timePicker_url,
                               body='{}',
                               status=401,
                               forcing_headers=headers)

        MockedTaskPanels.VERSION = '6.1.0'
        task = MockedTaskPanels(config)
        httpretty.enable(allow_net_connect=True)
        task.execute()
        httpretty.disable()


class TestTaskPanelsMenu(unittest.TestCase):
    """TaskPanelsMenu tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        config = Config(CONF_FILE)
        task = TaskPanelsMenu(config)

        self.assertEqual(task.config, config)

        self.assertEqual(len(task.panels_menu), 34)

        for entry in task.panels_menu:
            self.assertGreaterEqual(len(entry['index-patterns']), 0)


if __name__ == "__main__":
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>


import sys
import unittest

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sirmordred.config import Config
from sirmordred.sirmordred import logger, SirMordred

CONF_FILE = 'test.cfg'


class TestTaskRawDataCollection(unittest.TestCase):
    """Task tests"""

    def setUp(self):
        self.config = Config(CONF_FILE)
        self.sirmordred = SirMordred(self.config)

    def test_initialization(self):
        """Test whether attributes are initializated"""

        self.assertTrue(self.sirmordred.config, self.config)
        self.assertTrue(self.sirmordred.conf, self.config.get_conf())
        self.assertIsNotNone(self.sirmordred.grimoire_con)

    def test_check_es_access(self):
        """Test whether check_es_access properly works"""

        self.sirmordred = SirMordred(self.config)
        self.assertTrue(self.sirmordred.check_es_access())

    def test_check_es_access_es_collection_error(self):
        """Test whether an exception is thrown when es_collection is empty"""

        self.sirmordred = SirMordred(self.config)
        self.sirmordred.config.conf['es_collection']['url'] = ""
        with self.assertLogs(logger, level='ERROR') as cm:
            self.sirmordred.check_es_access()
            self.assertTrue(cm.output[-1], 'ERROR:sirmordred.sirmordred:Cannot connect to Elasticsearch: ')

    def test_check_es_access_es_es_enrichment_error(self):
        """Test whether an exception is thrown when es_enrichment is empty"""

        self.sirmordred = SirMordred(self.config)
        self.sirmordred.config.conf['es_enrichment']['url'] = ""
        with self.assertLogs(logger, level='ERROR') as cm:
            self.sirmordred.check_es_access()
            self.assertTrue(cm.output[-1], 'ERROR:sirmordred.sirmordred:Cannot connect to Elasticsearch: ')


if __name__ == "__main__":
    unittest.main(warnings='ignore')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>

import MySQLdb

host = "localhsot"
user = "root"
password = ""
name = "sortinghat_sh"
db = MySQLdb.connect(host, user, password, name)

cursor = db.cursor()
# get all uuid with more than 1 affiliation
cursor.execute("select * from (select uuid, count(*) as total from enrollments group by uuid) b where b.total>1;")


rows = cursor.fetchall()
for r in rows:
    uuid = str(r[0])
    # group by start,end date for uuids with more than 1 affilation
    cursor.execute("SELECT COUNT(*), start, end FROM enrollments WHERE uuid = '%s' GROUP BY start,end" % uuid)
    data = cursor.fetchone()
    total = data[0]
    start = data[1]
    end = data[2]
    if data[0] > 1:
        # conflict identified, gets organization names and uuid to compose the log
        cursor.execute("SELECT organizations.name FROM enrollments, organizations WHERE start = '%s' "
                       " AND end = '%s' AND uuid = '%s' AND organization_id = organizations.id" % (start, end, uuid))
        data = cursor.fetchall()
        orgs = []
        for o in data:
            orgs.append(o[0])
        orgs.sort()
        print("%s affiliation conflict with orgs %s" % (uuid, str(orgs)))

# disconnect from server
db.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#

import logging
import queue
import threading
import sys
import time

from datetime import datetime, timedelta

logger = logging.getLogger(__name__)


class TasksManager(threading.Thread):
    """
    Class to manage tasks execution

    All tasks in the same task manager will be executed in the same thread
    in a serial way.

    """

    # this queue supports the communication from threads to mother process
    COMM_QUEUE = queue.Queue()
    # to control if enrichment process are active
    NUMBER_ENRICH_TASKS_ON_LOCK = threading.Lock()
    NUMBER_ENRICH_TASKS_ON = 0
    # to control if identities process are active
    IDENTITIES_TASKS_ON_LOCK = threading.Lock()
    IDENTITIES_TASKS_ON = False

    def __init__(self, tasks_cls, backend_section, stopper, config, sortinghat_client, timer=0):
        """
        :tasks_cls : tasks classes to be executed using the backend
        :backend_section: perceval backend section name
        :config: config object for the manager
        """
        super().__init__(name=backend_section)  # init the Thread
        self.config = config
        self.tasks_cls = tasks_cls  # tasks classes to be executed
        self.tasks = []  # tasks to be executed
        self.backend_section = backend_section
        self.stopper = stopper  # To stop the thread from parent
        self.timer = timer
        self.thread_id = None
        self.client = sortinghat_client

    def add_task(self, task):
        self.tasks.append(task)

    def run(self):
        def __set_thread_id():
            self.thread_id = threading.get_ident()

        __set_thread_id()

        logger.debug('[%s] Task starts', self.backend_section)

        # Configure the tasks
        logger.debug(self.tasks_cls)
        for tc in self.tasks_cls:
            # create the real Task from the class
            task = tc(self.config, self.client)
            task.set_backend_section(self.backend_section)
            self.tasks.append(task)

        if not self.tasks:
            logger.debug('[%s] no tasks', self.backend_section)

        logger.debug('[%s] Tasks will be executed in this order: %s', self.backend_section, self.tasks)

        stop_task = False

        while not stop_task:
            for task in self.tasks:
                logger.debug('[%s] Tasks started: %s', self.backend_section, task)
                try:
                    task.execute()
                except Exception as ex:
                    logger.error("[%s] Exception in Task Manager %s", self.backend_section, ex, exc_info=True)
                    TasksManager.COMM_QUEUE.put(sys.exc_info())
                    raise
                logger.debug('[%s] Tasks finished: %s', self.backend_section, task)

            timer = self.__get_timer(self.backend_section)
            if timer > 0 and self.config.get_conf()['general']['update']:
                logger.info("[%s] sleeping for %s seconds ", self.backend_section, timer)
                time.sleep(timer)

            stop_task = self.stopper.is_set()

        logger.debug('[%s] Task is exiting', self.backend_section)

    def __get_timer(self, backend):
        if backend == "Global tasks":
            return self.timer

        update_hour = self.config.get_conf()['general'].get('update_hour', None)
        timer = self.timer
        if update_hour:
            now = datetime.now()
            next_date = now
            if now.hour >= update_hour:
                next_date = now + timedelta(days=1)
            update_hour_date = datetime(next_date.year, next_date.month, next_date.day, update_hour)
            timer = (update_hour_date - now).total_seconds()
        return timer
